import {
    TProcessWebResult,
    IWebPageProcessSettings,
    processWebPage as processWebPageContent,
    libTrackIsOn,
    TCost,
    TContext,
    newGuid,
    SURCHARGE,
    CREDIT_EXCHANGE_RATE
} from "encodeagent-helper-util";
import {
    getBrightDataInfo,
    HTTPERROR_400,
    ERROR_PARAMETER_MISSING
} from "encodeagent-helper-service";
import axios from "axios";
import { IPluginResult, calculateAICost } from "encodeagent-helper-ai-util";
import { logUsage as logUsageInternal } from "encodeagent-helper-lambda";

export type TProcessWebPageProps = {
    context: TContext;
    url: string;
    processContent: boolean;
    useProxy?: boolean;
    settings?: IWebPageProcessSettings & {
        logUsage?: boolean;
    };
};

export type TProcessWebPageResult = IPluginResult;

export const processWebPage = async (
    props: TProcessWebPageProps
): Promise<IPluginResult> => {
    const { url, processContent, context } = props;
    const { logUsage, ...settings } = props.settings ?? {};
    const useProxy = props.useProxy ?? true;
    let result: TProcessWebResult | string | undefined;

    if (useProxy) {
        const brightDataInfo = await getBrightDataInfo("WEB");
        if (!brightDataInfo) {
            return {
                type: "error",
                data: {
                    ...HTTPERROR_400,
                    type: ERROR_PARAMETER_MISSING,
                    detail: { paramName: "BrightDataInfo" }
                }
            };
        }

        const proxy = {
            protocol: "http",
            host: brightDataInfo.host,
            port: brightDataInfo.port,
            auth: {
                username: brightDataInfo.user,
                password: brightDataInfo.password
            }
        };

        libTrackIsOn() && console.log({ url, proxy });

        result = (
            await axios.get(url, {
                proxy
            })
        ).data;
    } else {
        result = (await axios.get(url)).data;
    }

    if (processContent) {
        libTrackIsOn() && console.log(result);
        result = processWebPageContent(url, result as string, settings ?? {});
    }

    if (logUsage) {
        const cost: TCost = calculateAICost({
            modelId: "encode-web-processor",
            defaultSurcharge: SURCHARGE,
            defaultCreditExchangeRate: CREDIT_EXCHANGE_RATE,
            inputTokens: 0,
            outputTokens: 1,
            additionalItems: []
        });

        await logUsageInternal({
            id: newGuid(),
            name: "process-web-page",
            context,
            cost,
            aiKeyProvider: "solution"
        });
    }

    return { type: "success", data: result };
};
